var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { ChangeDetectorRef, ComponentFactoryResolver, Directive, ElementRef, EventEmitter, forwardRef, HostListener, Injector, Input, Output, Renderer, TemplateRef, ViewContainerRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { fromEvent } from 'rxjs/Observable/fromEvent';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/filter';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/distinctUntilChanged';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/do';
import { RebirthUIConfig } from '../rebirth-ui.config';
import { PositionService } from '../position/positioning.service';
import { AutoCompletePopupComponent } from './auto-complete-popup.component';
var AutoCompleteDirective = AutoCompleteDirective_1 = (function () {
    function AutoCompleteDirective(elementRef, viewContainerRef, componentFactoryResolver, renderer, injector, positionService, rebirthUIConfig, changeDetectorRef) {
        this.elementRef = elementRef;
        this.viewContainerRef = viewContainerRef;
        this.componentFactoryResolver = componentFactoryResolver;
        this.renderer = renderer;
        this.injector = injector;
        this.positionService = positionService;
        this.rebirthUIConfig = rebirthUIConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.selectValue = new EventEmitter();
        this.placement = 'bottom-left';
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.valueChanges = this.registerInputEvent(elementRef);
        this.delay = rebirthUIConfig.autoComplete.delay;
        this.minLength = rebirthUIConfig.autoComplete.minLength;
        this.itemTemplate = rebirthUIConfig.autoComplete.itemTemplate;
        this.noResultItemTemplate = rebirthUIConfig.autoComplete.noResultItemTemplate;
        this.formatter = rebirthUIConfig.autoComplete.formatter;
        this.valueParser = rebirthUIConfig.autoComplete.valueParser;
    }
    AutoCompleteDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.subscription = this.valueChanges
            .subscribe(function (source) { return _this.onSourceChange(source); });
        var factory = this.componentFactoryResolver.resolveComponentFactory(AutoCompletePopupComponent);
        this.popupRef = this.viewContainerRef.createComponent(factory, this.viewContainerRef.length, this.injector);
        this.fillPopup();
        this.positionPopup();
        this.popupRef.instance.registerOnChange(function (item) {
            var value = _this.valueParser(item);
            _this.writeValue(value);
            _this.onChange(value);
            _this.hidePopup();
            _this.selectValue.emit(item);
        });
    };
    AutoCompleteDirective.prototype.writeValue = function (obj) {
        this.value = obj;
        this.writeInputValue(this.value);
    };
    AutoCompleteDirective.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    AutoCompleteDirective.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    AutoCompleteDirective.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.renderer.setElementProperty(this.elementRef.nativeElement, 'disabled', isDisabled);
        if (this.popupRef) {
            this.popupRef.instance.setDisabledState(isDisabled);
        }
    };
    AutoCompleteDirective.prototype.ngOnDestroy = function () {
        this.unSubscription();
    };
    AutoCompleteDirective.prototype.onBlur = function () {
        this.onTouched();
    };
    AutoCompleteDirective.prototype.onEscKeyup = function ($event) {
        this.hidePopup();
    };
    AutoCompleteDirective.prototype.onEnterKeyDown = function ($event) {
        this.hidePopup();
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.selectCurrentItem();
        }
    };
    AutoCompleteDirective.prototype.onArrowUpKeyDown = function ($event) {
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.prev();
        }
    };
    AutoCompleteDirective.prototype.onArrowDownKeyDown = function ($event) {
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.next();
        }
    };
    AutoCompleteDirective.prototype.onDocumentClick = function ($event) {
        var hostElement = this.elementRef.nativeElement;
        if ($event.target !== hostElement) {
            this.hidePopup();
        }
    };
    AutoCompleteDirective.prototype.onSourceChange = function (source) {
        if ((source && source.length) || this.noResultItemTemplate) {
            var pop = this.popupRef.instance;
            pop.reset();
            this.fillPopup(source, this.value);
            pop.isOpen = true;
            this.positionPopup();
            this.changeDetectorRef.markForCheck();
        }
    };
    AutoCompleteDirective.prototype.hidePopup = function () {
        if (this.popupRef) {
            this.popupRef.instance.isOpen = false;
        }
    };
    AutoCompleteDirective.prototype.positionPopup = function () {
        var targetElement = this.popupRef.location.nativeElement;
        var hostElement = this.elementRef.nativeElement;
        var clientRect = this.positionService.positionElements(hostElement, targetElement, this.placement, false);
        this.renderer.setElementStyle(targetElement, 'left', clientRect.left + "px");
        this.renderer.setElementStyle(targetElement, 'top', clientRect.top + "px");
    };
    AutoCompleteDirective.prototype.fillPopup = function (source, term) {
        var _this = this;
        var pop = this.popupRef.instance;
        pop.source = source;
        pop.term = term;
        ['formatter', 'itemTemplate', 'noResultItemTemplate', 'cssClass']
            .forEach(function (key) {
            if (_this[key] !== undefined) {
                pop[key] = _this[key];
            }
        });
    };
    AutoCompleteDirective.prototype.writeInputValue = function (value) {
        this.renderer.setElementProperty(this.elementRef.nativeElement, 'value', this.formatter(value || ''));
    };
    AutoCompleteDirective.prototype.unSubscription = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    };
    AutoCompleteDirective.prototype.onTermChange = function (term) {
        this.value = term;
        if (this.popupRef) {
            this.popupRef.instance.term = term;
        }
        this.onChange(term);
    };
    AutoCompleteDirective.prototype.registerInputEvent = function (elementRef) {
        var _this = this;
        return fromEvent(elementRef.nativeElement, 'input')
            .map(function (e) { return e.target.value; })
            .do(function (term) { return _this.onTouched(); })
            .filter(function (term) { return !_this.disabled && _this.onSearch && term.length >= _this.minLength; })
            .debounceTime(this.delay)
            .distinctUntilChanged()
            .do(function (term) { return _this.onTermChange(term); })
            .switchMap(function (term) { return _this.onSearch(term, _this); });
    };
    return AutoCompleteDirective;
}());
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], AutoCompleteDirective.prototype, "disabled", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AutoCompleteDirective.prototype, "cssClass", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], AutoCompleteDirective.prototype, "delay", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], AutoCompleteDirective.prototype, "minLength", void 0);
__decorate([
    Input(),
    __metadata("design:type", TemplateRef)
], AutoCompleteDirective.prototype, "itemTemplate", void 0);
__decorate([
    Input(),
    __metadata("design:type", TemplateRef)
], AutoCompleteDirective.prototype, "noResultItemTemplate", void 0);
__decorate([
    Input(),
    __metadata("design:type", Function)
], AutoCompleteDirective.prototype, "formatter", void 0);
__decorate([
    Input(),
    __metadata("design:type", Function)
], AutoCompleteDirective.prototype, "valueParser", void 0);
__decorate([
    Input(),
    __metadata("design:type", Function)
], AutoCompleteDirective.prototype, "onSearch", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], AutoCompleteDirective.prototype, "selectValue", void 0);
__decorate([
    HostListener('blur', []),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AutoCompleteDirective.prototype, "onBlur", null);
__decorate([
    HostListener('keydown.esc', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], AutoCompleteDirective.prototype, "onEscKeyup", null);
__decorate([
    HostListener('keydown.Enter', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], AutoCompleteDirective.prototype, "onEnterKeyDown", null);
__decorate([
    HostListener('keydown.ArrowUp', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], AutoCompleteDirective.prototype, "onArrowUpKeyDown", null);
__decorate([
    HostListener('keydown.ArrowDown', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], AutoCompleteDirective.prototype, "onArrowDownKeyDown", null);
__decorate([
    HostListener('document:click', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Event]),
    __metadata("design:returntype", void 0)
], AutoCompleteDirective.prototype, "onDocumentClick", null);
AutoCompleteDirective = AutoCompleteDirective_1 = __decorate([
    Directive({
        selector: '[reAutoComplete]',
        exportAs: 'autoComplete',
        host: {
            'autocomplete': 'off',
            'autocapitalize': 'off',
            'autocorrect': 'off'
        },
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(function () { return AutoCompleteDirective_1; }),
                multi: true
            }]
    }),
    __metadata("design:paramtypes", [ElementRef, ViewContainerRef,
        ComponentFactoryResolver, Renderer,
        Injector, PositionService,
        RebirthUIConfig, ChangeDetectorRef])
], AutoCompleteDirective);
export { AutoCompleteDirective };
var AutoCompleteDirective_1;
//# sourceMappingURL=auto-complete.directive.js.map